#----------------------------------------------------------------------------------#
#
# FILE:			__init__.py (HyperTail)
# AUTHOR:		Peter Quinn
# DATE:			02.02.2005
# VERSION:		1.50
#
# DESCRIPTION:	HyperTail control.
#
#----------------------------------------------------------------------------------#

from libdesklets.controls import Control, Interface, Permission
import commands, os, thread, re, select
from xml.sax.saxutils import escape
from stat import *

PATH		= 0
FILENAME	= 1
COLOUR		= 2
SIZE		= 3
FILETYPE	= 4


#----------------------------------------------------------------------------------#
#
# HyperTail Interface
#
class IHyperTail(Interface):

	refresh			= Permission.READ
	console_refresh	= Permission.READ
	console_output	= Permission.READ
	get_filetype	= Permission.READ
	get_status		= Permission.READ
	get_new_lines	= Permission.READ
	execute			= Permission.WRITE
	select_file		= Permission.WRITE
	filenames		= Permission.WRITE
	set				= Permission.WRITE
	unset			= Permission.WRITE
	set_regextra	= Permission.WRITE
	update			= Permission.WRITE
	add_file		= Permission.WRITE
	del_file		= Permission.WRITE
	set_colour		= Permission.WRITE
	suppress		= Permission.WRITE
	line_numbers	= Permission.WRITE
	digits			= Permission.WRITE
	padding			= Permission.WRITE
	buffer			= Permission.READWRITE


#----------------------------------------------------------------------------------#
#
# HyperTail Class
#
#
class HyperTail(Control, IHyperTail):

	#
	# HyperTail Constructor
	#
	def __init__(self):

		self._buffer			= []
		self._file				= []
		self._console_output	= []
		self._updated			= 0
		self._refresh			= 0
		self._console_refresh	= 0
		self._console_updated	= 0
		self._command			= ""
		self._console			= False
		self._regx				= ""
		self._regextra			= ""
		self._update_			= 600
		self._file_names		= True
		self._no_dest			= False
		self._no_null			= False
		self._no_empty			= False
		self._no_ttl			= False
		self._no_mac			= False
		self._suppress			= False
		self._line_numbers		= False
		self._digits			= 4
		self._padding			= False
		self._lineno			= 0
		self._file_pointer		= None
		self._new_lines			= 0
		
		Control.__init__(self)
		self._add_timer(4000, self.__get_data)


	#
	# set regular expression to be used for each new line
	#
	def __set_regx(self):
		
		self._regx = ""

		if self._no_mac:
			self._regx = self._regx + "|MAC=.*?\s"
		if self._no_ttl:
			self._regx = self._regx + "|TTL=.*?\s"
		if self._no_empty:
			self._regx = self._regx + "|[A-Z]+=\s"
		if self._no_null:
			self._regx = self._regx + "|[A-Z]+=0x00\s"
		if self._no_dest:
			self._regx = self._regx + "|DST=.*?\s"
		if self._regextra != "":
			self._regx = "%s|%s" % (self._regx, self._regextra)

		self._regx = re.sub("^\|", "", self._regx)


	#
	# Set pre-compiled regular expressions
	#
	def __set(self, value):

		if value == "mac":
			self._no_mac = True
		elif value == "ttl":
			self._no_ttl = True
		elif value == "empty":
			self._no_empty = True
		elif value == "null":
			self._no_null = True
		elif value == "dest":
			self._no_dest = True

		self.__set_regx()
	

	#
	# Unset pre-compiled regular expressions
	#
	def __unset(self, value):

		if value == "mac":
			self._no_mac = False
		elif value == "ttl":
			self._no_ttl = False
		elif value == "empty":
			self._no_empty = False
		elif value == "null":
			self._no_null = False
		elif value == "dest":
			self._no_dest = False

		self.__set_regx()


	#
	# Set extra regular expression
	#
	def __set_regextra(self, value):
		
		self._regextra = value
		self.__set_regx()


	#
	# Add a new file to the poll list
	#
	def __add_file(self, file):

		(path, colour) = file

		if os.path.exists(path):
			m = re.match(".*/(.*)$", path)

			if m:
				filename = m.group(1)
			else:
				filename = ""

			mode = os.stat(path)[ST_MODE]
			self._file.append((path, filename, colour, 0, S_ISFIFO(mode), 0, ""))


	#
	# Remove a file from the poll list
	#
	def __del_file(self, file):

		(path, colour) = file

		for i in range(len(self._file)):
			(file, filename, colour, size, filetype, pipe, pipetext) = self._file[i]

			if path == file:
				self._file = self._file[:i] + self._file[i + 1:]
				break


	#
	# Change the colour of a watch file
	#
	def __set_colour(self, file):

		(path, colour) = file

		for i in range(len(self._file)):
			(file, filename, col, size, filetype, pipe, pipetext) = self._file[i]

			if path == file:
				self._file[i] = (file, filename, colour, size, filetype, pipe, pipetext)
				break


	#
	# Set update
	#
	def __set_update(self, value):
		
		self._update_ = value


	#
	# Set filename prefix
	#
	def __set_filenames(self, value):

		self._file_names = value


	#
	# Get refresh status
	#
	def __get_refresh(self):

		return self._refresh


	#
	#
	#
	def __get_console_refresh(self):

		return self._console_refresh

		
	#
	# Get temporary buffer
	#
	def __get_buffer(self):

		return self._buffer


	#
	# Set empty line suppression
	#
	def __set_suppress(self, value):

		self._suppress = value


	#
	#
	#
	def __set_line_numbers(self, value):

		self._line_numbers = value


	#
	#
	#
	def __set_padding(self, value):

		self._padding = value

		
	#
	#
	#
	def __set_digits(self, value):

		self._digits = value


	#
	#
	#
	def __get_new_lines(self):

		return self._new_lines

		
	#
	#
	#
	def __get_filetype(self):

		(file, filename, colour, size, filetype, pipe, pipetext) = self._file[self._file_pointer]
		
		if filetype:
			return "Named Pipe"

		else:
			return "File"


	#
	#
	#
	def __get_status(self):

		(file, filename, colour, size, filetype, pipe, pipetext) = self._file[self._file_pointer]
	
		txt = commands.getoutput("if [ -r %s ]; then  echo \"yes\"; fi" % file)

		if txt == "yes":
			return "Accessable"

		else:
			return "Permission Denied"


	#
	#
	#
	def __select_file_(self, file):

		(path, colour) = file

		for i in range(len(self._file)):
			(file, filename, col, size, filetype, pipe, pipetext) = self._file[i]

			if path == file:
				self._file_pointer = i
				break


	#
	# Fill temporary buffer
	#
	def __set_buffer(self, arr):

		self._buffer = []

		for i in range(len(arr)):

			if arr[i]:
				(text, original, index, colour, lineno, path) = arr[i]
				if index != -1:
					self.__write_(original, index, self._file[index][COLOUR], lineno, path)

				else:
					self.__write_(" ", -1, "#FFFFFF", 0, "")
					
			else:
				self.__write_(" ", -1, "#FFFFFF", 0, "")


	#
	#
	#
	def __get_data(self):

		self._updated = 0
		self._refresh = 0

		thread.start_new_thread(self.__check_files, ())
		self._add_timer(150, self.__wait)

	
	#
	#
	#
	def __wait(self):

		if self._updated == 0:
			return 1

		elif self._updated == 1:
			if self._refresh == 1:
				self._update("refresh")

			self._add_timer(self._update_, self.__get_data)

		return 0


	#
	#
	#
	def __write_(self, txt, index, colour, lineno, path):

		txt = txt.strip()
		original = txt
	
		if self._file[index][PATH] and txt != "":
			txt = re.sub(self._regx, "", txt)

			line = ""
			
			if self._line_numbers:
				if self._padding == "none":
					num = "%d:"
				elif self._padding == "space":
					num = "%" + str(self._digits) + "d:"
				elif self._padding == "zero":
					num = "%." + str(self._digits) + "d:"
					
				line = num % lineno
				
			if self._file_names:
				txt = "%s[%s]%s" % (line, self._file[index][FILENAME], txt)
			else:
				txt = "%s%s" % (line, txt)

			txt = escape(txt)
	
		else:
			txt = " "

		self._buffer.append((txt, original, index, colour, lineno, path))


	def __execute(self, obj):

		(command, parameter, n) = obj

		if n != -1:
			self._console = True
			self._command = "%s %s" % (command, parameter)

		else:
			self._command = "%s %s" % (command, parameter)
			self._console = False
					
		self._console_updated = 0
		self._console_refresh = 0

		thread.start_new_thread(self.__execute_command, ())
		self._add_timer(200, self.__console_wait)

	
	#
	#
	#
	def __console_wait(self):

		if self._console_updated == 0:
			return 1

		elif self._console_updated == 1:
			if self._console:
				if self._console_refresh == 1:
					self._update("console_refresh")

		return 0

		
	#
	#
	#
	def __execute_command(self):

		str = commands.getoutput(self._command)
		self._console_output = str.split("\n")

		self._console_updated = 1
		self._console_refresh = 1


	#
	#
	#
	def __get_console_output(self):

		return self._console_output


	#
	#
	#
	def __pipe_thread(self, index, index2):

		fp = open(self._file[index][PATH], "r")
		txt = fp.read()
		fp.close()

		(filename, name, colour, size, filetype, pipe, pipetext) = self._file[index]
		self._file[index] = (filename, name, colour, size, filetype, 2, txt)

		return 0


	#
	# Check all files for changes, and if so, output their contents
	#
	def __check_files(self):

		self._buffer		= []
		self._new_lines		= 0
		
		for i in range(len(self._file)):
			(filename, name, colour, size, filetype, pipe, pipetext) = self._file[i]

			if filename:
				sz = os.stat(filename)[ST_SIZE]

				if filetype:
					if pipe == 0:
						self._file[i] = (filename, name, colour, size, filetype, 1, "")
						thread.start_new_thread(self.__pipe_thread, (i, i))

					elif pipe == 1:
						continue

					elif pipe == 2:
						lines = pipetext.split("\n")
						for line in lines:
							if pipetext == "\n" and self._suppress:
								continue
							self.__write_(line, i, colour, self._lineno, filename)
							self._lineno += 1
							self._new_lines += 1

						self._file[i] = (filename, name, colour, sz, filetype, 0, "")
						self._refresh = 1

				else:
					if sz > size:
						if size == 0:
							txt = commands.getoutput("tail -n 1 %s" % filename)
							if txt:
								if txt == "\n" and self._suppress:
									continue
								self.__write_(txt, i, colour, self._lineno, filename)
								self._lineno += 1
								self._new_lines += 1

						else:
							fp = open(filename, "rb")
							fp.seek(size)
	
							for line in fp.readlines():
								if line:
									if line == "\n" and self._suppress:
										continue
									self.__write_(line, i, colour, self._lineno, filename)
									self._lineno += 1
									self._new_lines += 1

							fp.close()

						self._file[i] = (filename, name, colour, sz, filetype, pipe, pipetext)
						self._refresh = 1

		self._updated = 1


	#
	# Interface
	#
	buffer			= property(	fget = __get_buffer, fset = __set_buffer,
								doc = "Get contents of buffer")
	refresh			= property(	fget = __get_refresh,
								doc = "HyperTail refresh trigger")
	execute			= property(	fset = __execute,
								doc = "Execute shell command")
	console_refresh	= property(	fget = __get_console_refresh,
								doc = "Console refresh trigger")
	console_output	= property(	fget = __get_console_output,
								doc = "Result from console command")
	filenames		= property(	fset = __set_filenames,
								doc = "Display/hide filenames")
	set				= property(	fset = __set,
								doc = "Set a pre-compiled regular expression")
	unset			= property(	fset = __unset,
								doc = "Unset a pre-compiled regular expression")
	set_regextra	= property(	fset = __set_regextra,
								doc = "Set a custom regular expression")
	update			= property(	fset = __set_update,
								doc = "Set the update time between polling files")
	add_file		= property(	fset = __add_file,
								doc = "Add a file to the file poll list")
	del_file		= property(	fset = __del_file,
								doc = "Remove a file from the poll list")
	set_colour		= property(	fset = __set_colour,
								doc = "Set the colour of a file watch")
	suppress		= property(	fset = __set_suppress,
								doc = "Turn on/off empty line suppression")
	line_numbers	= property(	fset = __set_line_numbers,
								doc = "Turn on/off line numbering")
	digits			= property(	fset = __set_digits,
								doc = "Set the amount of digits")
	padding			= property(	fset = __set_padding,
								doc = "Set the type of line number padding")
	get_filetype	= property(	fget = __get_filetype,
								doc = "Get the filetype of a file watch")
	get_status		= property(	fget = __get_status,
								doc = "Get status of a file watch")
	get_lines		= property(	fget = __get_new_lines,
								doc = "Get amount of new lines written")
	select_file		= property(	fset = __select_file_,
								doc = "Select file")


#----------------------------------------------------------------------------------#
	
def get_class(): return HyperTail

